@@ -64,11 +64,11 @@ import copy
 from operator import itemgetter
 import seqLister
 
-VERSION = "2.6.1"     # Semantic Versioning 2.0.0
+VERSION = "2.7.0"     # Semantic Versioning 2.0.0
 
 PROG_NAME = "lsseq"
 
-CACHE_EXT = [
+gCacheExtList = [
     "ass",
     "bgeo",
     "bgeo.gz",
@@ -85,7 +85,7 @@ CACHE_EXT = [
     "vdb.gz",
     "vdb.sc"
 ]
-MOV_EXT = [
+gMovieExtList = [
     "avi",
     "mov",
     "mp4",
@@ -93,7 +93,7 @@ MOV_EXT = [
     "mxf",
     "wmv"
 ]
-IMAGE_EXT = [
+gImageExtList = [
     "alpha",
     "als",
     "anim",
@@ -169,10 +169,23 @@ DATE_FORMAT_LIST = [
     '%Y%m%d-%H%M%S',
     '%y%m%d%H%M',    # Undocumented, but kept for compatibility
     '%Y%m%d%H%M',    # with earlier versions (v2.5.3 and earlier)
-    '%y%m%d%H%M.%S', # of lsseq. Note MMDDhhmm[.ss] is no longers
-    '%Y%m%d%H%M.%S'  # supported, thus not fully backward compatible.
+    '%y%m%d%H%M.%S', # of lsseq. Note MMDDhhmm[.ss] is no longer
+    '%Y%m%d%H%M.%S'  # supported, thus not FULLY backward compatible.
 ]
 
+# Support for --globalSortByTime feature.
+#
+gTimeList = []
+gImageDictionary = {}
+gCacheDictionary = {}
+gMoviesDictionary = {}
+
+# Array indices for timeList (used in "listSeqDir()") and gTimeList
+#
+DICTKEY = 0
+MTIME = 1
+TRAVERSEDPATH = 2
+
 PATH_NOPREFIX = 0
 PATH_ABS = 1
 PATH_REL = 2
@@ -218,6 +231,9 @@ FRAMENUM = 1
 # Thus "loose" is only defined as the use of "_" over and above the
 # far more desirable strict case of only allowing ".".
 #
+# Actually the following global variable is never used in the code.
+# But keeping it here for the important discussion above.
+#
 LOOSE_SEP = "_"
 
 def isFrameNum(f) :
@@ -248,9 +264,6 @@ def readByteShortForm(numBytes) :
 # Splits up a filename by the dots in the name.
 #
 def splitFileComponents(filename) :
-    global IMAGE_EXT
-    global CACHE_EXT
-
     fileComponents = filename.split(".")
 
     # A file with no extension.
@@ -262,8 +275,8 @@ def splitFileComponents(filename) :
     # and join them before returning result.
     # Note: use of lower() allows us to ignore case of extensions.
     #
-    if (".".join(fileComponents[-2:]).lower() in IMAGE_EXT) \
-	    or (".".join(fileComponents[-2:]).lower() in CACHE_EXT) :
+    if (".".join(fileComponents[-2:]).lower() in gImageExtList) \
+	    or (".".join(fileComponents[-2:]).lower() in gCacheExtList) :
 
         # If we found a match, join the last two items,
         # into the second last slot, then and delete the last one.
@@ -282,9 +295,6 @@ def splitFileComponents(filename) :
 #
 def seqSplit(filename, args) :
 
-    global IMAGE_EXT
-    global CACHE_EXT
-    global LOOSE_SEP
     fileComponents = splitFileComponents(filename)
 
     # A file with no extension.
@@ -295,8 +305,8 @@ def seqSplit(filename, args) :
     # Test if image or cache sequence.
     # Note: use of lower() allows us to ignore case of extensions.
     #
-    if (fileComponents[-1].lower() in IMAGE_EXT) \
-            or (fileComponents[-1].lower() in CACHE_EXT) :
+    if (fileComponents[-1].lower() in gImageExtList) \
+            or (fileComponents[-1].lower() in gCacheExtList) :
 
         if not args.strictSeparator :
             looseFileComponents = fileComponents[-2].split("_")
@@ -324,13 +334,12 @@ def seqSplit(filename, args) :
 # Return true if and only if filename is a movie file.
 #
 def isMovie(filename) :
-    global MOV_EXT
     fileComponents = filename.split(".")
 
     # Note: use of lower() allows us to ignore case of extensions.
     #
     return len(fileComponents) > 1 \
-        and fileComponents[-1].lower() in MOV_EXT
+        and fileComponents[-1].lower() in gMovieExtList
 
 
 # Split the filename dictionary KEY into (<imagename>, "", <ext>)
@@ -355,7 +364,7 @@ def splitImageName(filenameKey) :
 def isCache(keyName) :
     splitName = splitImageName(keyName)
     # use of lower() allows us to ignore case of extensions.
-    return splitName[-1].lower() in CACHE_EXT
+    return splitName[-1].lower() in gCacheExtList
 
 # Reconstruct the imagename with the frame number
 # from the dictionary key..
@@ -706,10 +715,12 @@ def stripDotFiles(dirContents, stripIt) :
 # 
 def listSeqDir(dirContents, path, listSubDirs, args, traversedPath) :
 
-    # Array indices for "timeList" used below.
+    # Declare global variables since they might be modified by this function.
     #
-    DICTKEY = 0
-    MTIME = 1
+    global gTimeList
+    global gImageDictionary
+    global gCacheDictionary
+    global gMoviesDictionary
 
     # Stash the current working dir, to come back to and the end
     # of this function. I.e.; we need to push and pop the current
@@ -902,31 +913,53 @@ def listSeqDir(dirContents, path, listSubDirs, args, traversedPath) :
                 timeList.append((k, int(time)))
 
     if args.sortByMTime :
-        timeList.sort(key=itemgetter(MTIME)) # Sorts by time.
-        # Note: ls -t prints newest first; ls -tr is newest last.
-        if not args.reverseListing :
-            timeList.reverse()
-        for seq in timeList :
-            if args.cutoffTime != None :
-                if args.cutoffTime[0] == 'before' :
-                    if seq[MTIME] > args.cutoffTime[1] :
-                        continue
-                else : # Guaranteed to be 'since'
-                    if seq[MTIME] < args.cutoffTime[1] :
-                        continue
-            if isMovie(seq[DICTKEY]) :
-                if args.prependPath != PATH_NOPREFIX :
-                    sys.stdout.write(traversedPath)
-                print(seq[DICTKEY])
-                somethingWasPrinted = True
-            elif isCache(seq[DICTKEY]) :
-                cacheDictionary[seq[DICTKEY]].sort(key=itemgetter(FRAME_NUM, FRAME_PADDING))
-                printSeq(seq[DICTKEY], cacheDictionary[seq[DICTKEY]], args, traversedPath)
-                somethingWasPrinted = True
-            else :
-                imageDictionary[seq[DICTKEY]].sort(key=itemgetter(FRAME_NUM, FRAME_PADDING))
-                printSeq(seq[DICTKEY], imageDictionary[seq[DICTKEY]], args, traversedPath)
-                somethingWasPrinted = True
+        if args.globalSortByTime :
+            #
+            # Append sequences to the global list for later printing.
+            # then print nothing and continue below (recursive descent,
+            # or processing other directory contents). Need to extend
+            # the tuple in gTimeList to include "traversedPath" since
+            # that string won't be availabe when we finally emerge from
+            # listSeqDir() in main(). Also need copies of the dictionaries 
+            # saved globally for use in main(), but the key needs to 
+            # be extended to include traversedPath to get unique keys
+            # globally.
+            #
+            for seq in timeList :
+                gTimeList.append( (seq[DICTKEY], seq[MTIME], traversedPath) )
+                gDictKey = traversedPath + '/' + seq[DICTKEY]
+                if isMovie(seq[DICTKEY]) :
+                    gMoviesDictionary[gDictKey] = moviesDictionary[seq[DICTKEY]]
+                elif isCache(seq[DICTKEY]) :
+                    gCacheDictionary[gDictKey] = cacheDictionary[seq[DICTKEY]]
+                else :
+                    gImageDictionary[gDictKey] = imageDictionary[seq[DICTKEY]]
+        else :
+            timeList.sort(key=itemgetter(MTIME)) # Sorts by time.
+            # Note: ls -t prints newest first; ls -tr is newest last.
+            if not args.reverseListing :
+                timeList.reverse()
+            for seq in timeList :
+                if args.cutoffTime != None :
+                    if args.cutoffTime[0] == 'before' :
+                        if seq[MTIME] > args.cutoffTime[1] :
+                            continue
+                    else : # Guaranteed to be 'since'
+                        if seq[MTIME] < args.cutoffTime[1] :
+                            continue
+                if isMovie(seq[DICTKEY]) :
+                    if args.prependPath != PATH_NOPREFIX :
+                        sys.stdout.write(traversedPath)
+                    print(seq[DICTKEY])
+                    somethingWasPrinted = True
+                elif isCache(seq[DICTKEY]) :
+                    cacheDictionary[seq[DICTKEY]].sort(key=itemgetter(FRAME_NUM, FRAME_PADDING))
+                    printSeq(seq[DICTKEY], cacheDictionary[seq[DICTKEY]], args, traversedPath)
+                    somethingWasPrinted = True
+                else :
+                    imageDictionary[seq[DICTKEY]].sort(key=itemgetter(FRAME_NUM, FRAME_PADDING))
+                    printSeq(seq[DICTKEY], imageDictionary[seq[DICTKEY]], args, traversedPath)
+                    somethingWasPrinted = True
     elif args.cutoffTime != None :
         timeList.sort(key=itemgetter(DICTKEY)) # Sorts by name.
         if args.reverseListing :
@@ -1011,16 +1044,12 @@ def main() :
             pass
     sys.excepthook = new_hook
 
-    global IMAGE_EXT
-    global MOV_EXT
-    global CACHE_EXT
-    global PATH_ABS
-    global PATH_REL
-    global LIST_ALLFILES
-    global LIST_ONLYSEQS
-    global LIST_ONLYIMGS
-    global LIST_ONLYMOVS
-    global LIST_ONLYCACHES
+    # These global variables might be changed below depending on
+    # whether some environment variables are set.
+    #
+    global gImageExtList
+    global gMovieExtList
+    global gCacheExtList
 
     # To help with argparse.
     #
@@ -1200,6 +1229,11 @@ def main() :
     p.add_argument("--recursive", "-R", action="store_true",
         dest="isRecursive", default=False,
         help="list subdirectories recursively")
+    p.add_argument("-t", "--sortByTime", action="store_true",
+        dest="sortByMTime", default=False,
+        help="sort by modification time, the default comparison \
+        time is between the most recently modified (newest) frames \
+        in each sequence. (see --time) (see ls(1))")
     p.add_argument("--time", action="store", type=str,
         dest="timeCompare",
         help="which frame in the sequence to use to compare times \
@@ -1207,11 +1241,6 @@ def main() :
         for 'FRAME_AGE' are 'oldest', 'median' and 'newest' \
         [default: 'newest']", metavar="FRAME_AGE", default="newest",
         choices=("oldest", "median", "newest"))
-    p.add_argument("-t", action="store_true",
-        dest="sortByMTime", default=False,
-        help="sort by modification time, the default comparison \
-        time is between the most recently modified (newest) frames \
-        in each sequence. (see --time) (see ls(1))")
     p.add_argument("--onlyShow", action="store", type=str, nargs=2,
         dest="cutoffTime",
         help="where TENSE is either 'before' or 'since'; only list sequences \
@@ -1221,6 +1250,13 @@ def main() :
         The optional '-hh' (hours), 'mm' (minutes) or 'ss' (seconds) \
         default to zero if not specified.",
         metavar=("TENSE", "[CC]YYMMDD[-hh[mm[ss]]]"))
+    p.add_argument("--globalSortByTime", action="store_true",
+        dest="globalSortByTime", default=False,
+        help="when using either \
+        --prependPathAbs or --prependPathRel then this option will sort ALL \
+        sequences by time compared to each other, as opposed to only sorting \
+        sequences by time within their common directory. If the above contitions \
+        are NOT met, then this option is simply ignored.")
     p.add_argument("--silent", "--quiet", action="store_true",
         dest="silent", default=False,
         help="suppress errors and warnings")
@@ -1240,7 +1276,7 @@ def main() :
     elif tmpOICExt != None and tmpOICExt != "" :
         tmpExtList = tmpOICExt.split(":")
     else :
-        tmpExtList = IMAGE_EXT
+        tmpExtList = gImageExtList
     #
     # Using a set below removes duplicates if they exist.
     #
@@ -1253,7 +1289,7 @@ def main() :
     for e in tmpExtList :
         tmpExtSet.add(e.lower())
     tmpExtList = sorted(tmpExtSet)
-    IMAGE_EXT = copy.deepcopy(tmpExtList)
+    gImageExtList = copy.deepcopy(tmpExtList)
 
     tmpExt = os.getenv("LSSEQ_MOV_EXTENSION")
     tmpOICExt = os.getenv("OIC_MOV_EXTENSION")
@@ -1262,13 +1298,13 @@ def main() :
     elif tmpOICExt != None and tmpOICExt != "" :
         tmpExtList = tmpOICExt.split(":")
     else :
-        tmpExtList = MOV_EXT
+        tmpExtList = gMovieExtList
     # Using a set like this removes duplicates if they exist.
     tmpExtSet = set([])
     for e in tmpExtList :
         tmpExtSet.add(e.lower())
     tmpExtList = sorted(tmpExtSet)
-    MOV_EXT = copy.deepcopy(tmpExtList)
+    gMovieExtList = copy.deepcopy(tmpExtList)
 
     tmpExt = os.getenv("LSSEQ_CACHE_EXTENSION")
     tmpOICExt = os.getenv("OIC_CACHE_EXTENSION")
@@ -1277,13 +1313,13 @@ def main() :
     elif tmpOICExt != None and tmpOICExt != "" :
         tmpExtList = tmpOICExt.split(":")
     else :
-        tmpExtList = CACHE_EXT
+        tmpExtList = gCacheExtList
     # Using a set like this removes duplicates if they exist.
     tmpExtSet = set([])
     for e in tmpExtList :
         tmpExtSet.add(e.lower())
     tmpExtList = sorted(tmpExtSet)
-    CACHE_EXT = copy.deepcopy(tmpExtList)
+    gCacheExtList = copy.deepcopy(tmpExtList)
 
     #
     # Respond to arguments set by user and/or set up variables
@@ -1291,12 +1327,14 @@ def main() :
     #
 
     if args.printImgExtensions :
-        print("Note:", PROG_NAME, "also recognizes all extensions below when uppercase.")
-        extList = ":".join(IMAGE_EXT)
+        print(PROG_NAME,
+            ": note: ", PROG_NAME, " also recognizes all extensions below when uppercase.",
+            sep='')
+        extList = ":".join(gImageExtList)
         print("LSSEQ_IMAGE_EXTENSION:", extList)
-        extList = ":".join(MOV_EXT)
+        extList = ":".join(gMovieExtList)
         print("LSSEQ_MOV_EXTENSION:", extList)
-        extList = ":".join(CACHE_EXT)
+        extList = ":".join(gCacheExtList)
         print("LSSEQ_CACHE_EXTENSION:", extList)
         sys.exit(0)
 
@@ -1353,6 +1391,19 @@ def main() :
         import time
         args.cutoffTime[1] = int(time.mktime(timeData.timetuple())) # Epoch time
 
+    # args.globalSortByTime is only used in the code above to defer printing
+    # the sequences until the end, after ALL the sequences been collected into
+    # a common dictionary. We check the validity of the other command-line
+    # options to determine if this flag should be set (see help description above).
+    # Also we need to turn on args.sortByMTime so that some code above gets
+    # excuted which captures times etc.
+    #
+    if args.globalSortByTime :
+        if args.prependPath == PATH_NOPREFIX :
+            args.globalSortByTime = False
+        else :
+            args.sortByMTime = True # Needed to engage code to capture times
+
     # Now the meat and potatoes.
     # The following logic attempts to mimic the behavior
     # of /bin/ls as closely as possible.
@@ -1427,5 +1478,34 @@ def main() :
             passedPath = os.getcwd() + "/"
         listSeqDir(args.files, ".", True, args, passedPath)
 
+
+    # If we need to print the sequences globally sorted by time,
+    # we do it here, note that in this case nothing will have been
+    # printed so far. Also we know that args.prependPath != PATH_NOPREFIX
+    # since this option is turned OFF if this is not the case.
+    #
+    if args.globalSortByTime :
+        gTimeList.sort(key=itemgetter(MTIME)) # Sorts by time.
+        # Note: ls -t prints newest first; ls -tr is newest last.
+        if not args.reverseListing :
+            gTimeList.reverse()
+        for seq in gTimeList :
+            if args.cutoffTime != None :
+                if args.cutoffTime[0] == 'before' :
+                    if seq[MTIME] > args.cutoffTime[1] :
+                        continue
+                else : # Guaranteed to be 'since'
+                    if seq[MTIME] < args.cutoffTime[1] :
+                        continue
+            gDictKey = seq[TRAVERSEDPATH] + '/' + seq[DICTKEY]
+            if isMovie(seq[DICTKEY]) :
+                print(gDictKey)
+            elif isCache(seq[DICTKEY]) :
+                gCacheDictionary[gDictKey].sort(key=itemgetter(FRAME_NUM, FRAME_PADDING))
+                printSeq(seq[DICTKEY], gCacheDictionary[gDictKey], args, seq[TRAVERSEDPATH])
+            else :
+                gImageDictionary[gDictKey].sort(key=itemgetter(FRAME_NUM, FRAME_PADDING))
+                printSeq(seq[DICTKEY], gImageDictionary[gDictKey], args, seq[TRAVERSEDPATH])
+
 if __name__ == '__main__' :
     main()
